/************************************************************************
 *
 * this process makes the different cPCI slowcontrol interface (fans and temperatures)
 * ioctl's available from the shell level
 *  
 * Author : Jean-Francois Huppert & Pascal Vincent
 *          LPNHE-Paris
 * Modification : Panazol jean-luc
 * e-mail : huppert@lpnhep.in2p3.fr, vincentp@in2p3.fr 
 * Date   : 5 june 2001
 * Modif  : J-Luc PANAZOL oct 2007
 ************************************************************************
 * CtrlSecurityPilot.c
 */

#include <sys/types.h> 
#include <sys/stat.h>
#include <fcntl.h> 
#include <unistd.h>
#include <sys/ioctl.h>   
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/mman.h>
#include <errno.h>
#include <ctype.h>
#include <time.h>
#include <termios.h>

#include "HESSDAQDefine.h"
#include "HESSDAQControl.h"
#include "cpu_def.h"    
#include "ctrlsecurity_def.h"     

#define __MAIN__
#include "CtrlSecurityPilot.h"
#undef __MAIN__

/************************************************************************/

void List_Defaults(int scripts) {

  int  ret_val;
  char mess[256];

  printf("\n Defaults Settings :\n");
  printf("      -- BLOCK FILE   used :  %s\n",Blk_File_name);
  if(scripts) printf("      -- SCRIPT_FILES used :  none\n");
  else printf("      -- SCRIPT_FILES used :  %s\n",FIFO_SCRIPT_LIST);

  ret_val = ioctl(file_desc,GET_SLOWCONTROL_READ_MODE,0);
  printf("      -- Input/Output mode for BLOCK transfer : %x (hex)\n",ret_val);
  if(ret_val == 0) strcpy(mess,"Programmed_IO");
  else strcpy(mess,"DMA");
  strcpy(&mess[20],"Programmed_IO");
  printf("         %s Read, %s Write\n",mess,&mess[20]);
}

/************************************************************************/

int main(int argc, char *argv[]) {

  int 
    i,
    j,
    gbf,
    display,
    nval = 0,
    nsscan,
    ret_val,
    scripts,
    nscripts,
    scr_level,
    block_num,
    count = 0;

  time_t
    t1,
    t2;

  char 
    mess[256],
    *pt;

  FILE * ScF;
  int indesc[SCRIPTS_RECURSIVITY];

  struct termios
    confg_term,
    sauve_term;

  static int
    last_val;

//--------------------------------------------------------------------

  setbuf(stdin, NULL);
  printf("\n\n");

  if ((file_desc = open(FAN_DEVICE_NAME, O_RDWR)) < 0) {
    printf ("Error %d Can't open device file: %s\n",file_desc,FAN_DEVICE_NAME);
    exit(1);
  }

  strncpy(Blk_File_name, FIFO_BLOCK_FILE_NAME, sizeof(Blk_File_name));
  if (argc > 1) {
    j = 0;
    for(i = 0; i < strlen(argv[1]); i++) {
      if(*(argv[1]+i) != ' ') j++;
      if(*(argv[1]+i) == '-') {
	if(*(argv[1]+i+1) == '?') {
	  printf("\nusage is : %s [ FIFO_BLOCK_FILE_NAME ]\n\n",argv[0]);
	  exit(0);
	}
	printf("\nerror : Block_File_Name '%s' should not start with '-' !\n\n",argv[1]);
	exit(1);
      }
    }
    if(j < 8) {
      printf("\nerror : Block_File_Name '%s' is too short !\n\n",argv[1]);
      exit(1);
    }
    strncpy(Blk_File_name,argv[1],sizeof(Blk_File_name));
  }

  gbf = Get_Fifo_BLks(Blk_File_name);
  nscripts = 0;
  scripts = Read_Scripts_List(&nscripts);

  val = 0;
  last_val = val;
  lastp64 = 0;
  last_deleted = 0;
  last_ideleted = -1;
  block_num = 0;
  scr_level = 0;
  strcpy(Current_Script_name,"stdin");
  fifo_pattern = 0xf;
  display = 2;

  while(val>=0) {

    if(display) {
      printf("\n\n*******  SLOWCONTROL_PILOT COMMANDS (parameters in hexa)  *******\n");
      if(display == 2) List_Defaults(scripts);
      printf("\n");
      printf("************** Temperature ***************\n");
      printf("   0  Read temperature control register\n");
      printf("   1  Write temperature control register  data(31..0)\n");
      printf("   2  Read temperature status 1 register\n");
      printf("   3  Read temperature status 2 register\n");
	  printf("   4  Read Mux channel [4..0]\n");
      printf("   5  Write Mux channel [4..0]\n");
      printf("   6  Read Mux channel value [7..0]\n\n");
      printf("************** FAN ***********************\n");
      printf("   7  Read FAN control register\n");
      printf("   8  Write FAN control register           data(1..0)\n");
      printf("   9  Read FAN status register\n\n");
      printf("************** LED ***********************\n");
	  printf("   a  Read LED control register\n");
      printf("   b  Write LED control register           data(1..0)\n");
      printf("   c  Read LED status register\n\n");
      printf("************** GPIO **********************\n");
      printf("   d  Read GPIO status register\n");
      printf("   e  Read GPIO report register            data(31..0)\n");
      printf("   f  Read GPIO update register\n");
	  printf("  10  Write GPIO update register\n");
	  printf("  11  Write GPIO control register (not used)\n");
	  printf("  13  Write GPIO password register (not yet implemented)\n\n");
	  
		
      printf("************** Photodiode ****************\n");
	  printf("  14  Read Photodiode control register\n");
      printf("  15  Write Photodiode control register    data(31..0)\n");
      printf("  16  Read Photodiode status 1 register\n");
	  printf("  17  Read Photodiode status 2 register\n");
      printf("  18  Write configure CFR register         no data required)\n");
	  printf("  19  Read : Read CFR register\n");
	  printf("  1a  Write : Read CFR register            data(11..0)\n");
	  printf("  1b  Read : Photodiode value register     data(11..0)\n\n");


	  printf("************** Misc *********************\n");
      printf("  1c  Read Test register\n");
      printf("  1d  Write Test register                 data(31..0)\n");
      printf("  1e  Read Safety FPGA version register\n");
      printf("  1f  Read General control register\n");
      printf("  20  Write General control register\n");
      printf("  21  Read General status register (not used)\n");
      printf("  22  Read Interrupt test register[3..0]\n");
      printf("  23  Write Interrupt test register[3..0]\n\n");

      printf("************** BAR 0 Commands ****************\n");
      printf("  24  Read circular buffer temperature : (bar0)\n");
      printf("  25  Read 16 zone Fan DAC register : (bar0)\n");
      printf("  26  Write 16 zone Fan DAC register : (bar0)     data(7..0)\n\n");
      printf("  27  Read 20 calibration Led registers : (bar0)\n");
      printf("  28  Write 20 calibration Led registers : (bar0) data(7..0)\n\n");

      printf("************************************************\n");
      printf("  3a  REPEAT_LAST_COMMAND                         [ntimes]\n\n");
      printf("  3b  LIST_SCRIPT_FILES\n");
      printf("  2c  EXECUTE_SCRIPT_FILE                         [script_file]\n");
      printf("  2d  SCRIPT_PAUSE                                [nseconds]\n\n");
      printf("  -1  EXIT or QUIT_SCRIPT\n");
      printf("   ?  SHOW_COMMAND_MENU\n\n");
      printf(" ... your choice : ");
	 
    }
    else {
      if(scr_level == 0) printf("\n      press ? to get command menu : ");
    }

    param64 = 0;
    param2_64 =0;
    display = 0;
    param   = 0;
    param2  =0;
    val     = 0;

    pt = fgets(prm,64,stdin);
    if(pt == NULL) strcpy(prm,"-1 ");
    else 
    if(scr_level > 0) printf("\n command : %s",prm);
      
    nsscan = sscanf(prm,"%x %Lx %Lx",&val,&param64,&param2_64); 
    if(nsscan <= 0) val = SHOW_MENU;
    printf("\n ---- ioctl return : ");

    if (val == 0x3a) {
      if(nsscan > 1) count = (unsigned int)param64;
      else count = 1;
      val = last_val;
      param64 = lastp64;
    }
    else
    if((val >= 0) && (val < SHOW_MENU)) {
      nval = nsscan;
      last_val = val;
      if(nval > 1) lastp64 = param64;
      count = 1;
    }

    ret_val = -1;
    printf("param=%Lx param2=%Lx\n",param64,param2_64);
    param = (unsigned int)param64;
    param2 = (unsigned int)param2_64;
    printf("param =%x param2=%x\n",param,param2);
   switch (val) {

//--------------------------------------------------------------------
      case 0:
		  while(count--) {cPCI_Temp_Ctrl_Read(file_desc);};
	break;
//--------------------------------------------------------------------
      case 1:
	if(nval <= 1) {
          printf("\n\n argument missing : data\n");
	  count = 0;
	  break;
	}
	while(count--) {cPCI_Temp_Ctrl_Write(file_desc, param);};
	break;
//--------------------------------------------------------------------
      case 2:
	while(count--) {cPCI_Temp_Status1_Read(file_desc);};	
	break;
//--------------------------------------------------------------------
      case 3:
	while(count--) {cPCI_Temp_Status2_Read(file_desc);};
	break;
//--------------------------------------------------------------------
      case 4:
	while(count--) {cPCI_Temp_MUX_Read(file_desc);};
	break;
//--------------------------------------------------------------------
      case 5:
	if(nval <= 1) {
          printf("\n\n argument missing : data\n");
	  count = 0;
	  break;
	}
	while(count--) {cPCI_Temp_MUX_Write(file_desc, param);};
	break;
//--------------------------------------------------------------------
      case 6:
	while(count--) {cPCI_Temp_MUX_VALUE_Read(file_desc);};	
	break;
//--------------------------------------------------------------------
	  case 0x7:
	while(count--) {cPCI_Fan_Ctrl_Read(file_desc);};
	break;
//--------------------------------------------------------------------
      case 0x8:
	if(nval <= 1) {
          printf("\n\n argument missing : data\n");
	  count = 0;
	  break;
	}
	while(count--) {cPCI_Fan_Ctrl_Write(file_desc, param);};
	break;
//--------------------------------------------------------------------
      case 0x9:
	while(count--) {cPCI_Fan_Status_Read(file_desc);};	
	break;

	//--------------------------------------------------------------------

      case 0xa:
	while(count--) {cPCI_Led_Ctrl_Read(file_desc);};
	break;
//--------------------------------------------------------------------
      case 0xb:
	if(nval <= 1) {
          printf("\n\n argument missing : data\n");
	  count = 0;
	  break;
	}
	while(count--) {cPCI_Led_Ctrl_Write(file_desc, param);};
	break;
//--------------------------------------------------------------------
      case 0xc:
	while(count--) {cPCI_Led_Status_Read(file_desc);};	
	break;
//--------------------------------------------------------------------
      case 0xd:
	while(count--) {cPCI_GPIO_Status_Read(file_desc);};
	break;
	
//--------------------------------------------------------------------
      case 0xe:
	while(count--) {cPCI_GPIO_Report_Read(file_desc);};
	break;
//--------------------------------------------------------------------
      case 0xf:
	while(count--) {cPCI_GPIO_Update_Read(file_desc);};	
	break;

//--------------------------------------------------------------------
      case 0x10:
	while(count--) {cPCI_GPIO_Update_Write(file_desc, param);};	
	break;

//--------------------------------------------------------------------
      case 0x11:
	if(nval <= 1) {
          printf("\n\n argument missing : data\n");
	  count = 0;
	  break;
	}
	while(count--) {cPCI_GPIO_Ctrl_Write(file_desc, param);};
	break;
//--------------------------------------------------------------------
      case 0x13:
	if(nval <= 1) {
          printf("\n\n argument missing : data\n");
	  count = 0;
	  break;
	}
	while(count--) {cPCI_GPIO_Password_Write(file_desc, param);};
	break;
//--------------------------------------------------------------------
 
	  case 0x14:
		  while(count--) {cPCI_Photodiode_Ctrl_Read(file_desc);};
	break;
//--------------------------------------------------------------------
      case 0x15:
	if(nval <= 1) {
          printf("\n\n argument missing : data\n");
	  count = 0;
	  break;
	}
	while(count--) {cPCI_Photodiode_Ctrl_Write(file_desc, param);};
	break;
//--------------------------------------------------------------------
      case 0x16:
	while(count--) {cPCI_Photodiode_Status1_Read(file_desc);};	
	break;
//--------------------------------------------------------------------
   case 0x17:
	while(count--) {cPCI_Photodiode_Status2_Read(file_desc);};
	break;

//--------------------------------------------------------------------
   case 0x18:
	if(nval <= 1) {
          printf("\n\n argument missing : data\n");
	  count = 0;
	  break;
	}
	while(count--) {cPCI_Config_CFR_Write(file_desc, param);};
	break;
	
//--------------------------------------------------------------------

      case 0x19:
	while(count--) {cPCI_CFR_Read(file_desc);};
	break;
//--------------------------------------------------------------------
	  case 0x1a:
	if(nval <= 1) {
          printf("\n\n argument missing : data\n");
	  count = 0;
	  break;
	}
	while(count--) {cPCI_CFR_Write(file_desc, param);};
	break;
	
//--------------------------------------------------------------------
      case 0x1b:
	while(count--) {cPCI_Photodiode_Value_Read(file_desc);};
	break;
	
//--------------------------------------------------------------------
    case 0x1c:
	while(count--) {cPCI_TestRegister_Read(file_desc);};
	break;

//--------------------------------------------------------------------
      case 0x1d:
	if(nval <= 1) {
          printf("\n\n argument missing : data\n");
	  count = 0;
	  break;
	}
	while(count--) {cPCI_Test_Register_Write(file_desc, param);};
	break;
//--------------------------------------------------------------------
      case 0x1e:
	while(count--) {cPCI_FPGA_Version_Read(file_desc);};
	break;
//--------------------------------------------------------------------
      case 0x1f:
	while(count--) {cPCI_General_Ctrl_Read(file_desc);};
	break;
//--------------------------------------------------------------------
      case 0x20:
	if(nval <= 1) {
          printf("\n\n argument missing : data\n");
	  count = 0;
	  break;
	} 
	while(count--) {cPCI_General_Ctrl_Write(file_desc, param);};
	break;
//--------------------------------------------------------------------
      case 0x21:
	while(count--) {cPCI_General_Status_Read(file_desc);};
	break;
//--------------------------------------------------------------------
      case 0x22:
	while(count--) {cPCI_Interrupt_Test_Read(file_desc);};
	break;
//--------------------------------------------------------------------
      case 0x23:
	if(nval <= 1) {
          printf("\n\n argument missing : data\n");
	  count = 0;
	  break;
	}
	while(count--) {cPCI_Interrupt_Test_Write(file_desc, param);};
	break;
//--------------------------------------------------------------------
      case 0x24:
	if(nval <= 1) {
          printf("\n\n argument missing : data\n");
	  count = 0;
	  break;
	}
	while(count--) {cPCI_Last_Temperature_Value_Read(file_desc, param);};
	break;
//--------------------------------------------------------------------
      case 0x25:
	if(nval <= 1) {
          printf("\n\n argument missing : data\n");
	  count = 0;
	  break;
	}
	while(count--) {
                   cPCI_Zone_Fan_DAC_Register_Read(file_desc, param);
                   cPCI_Fan_Status_Read(file_desc);
    };
	break;
//--------------------------------------------------------------------
      case 0x26:
	if(nval <= 1) {
          printf("\n\n argument missing : data\n");
	  count = 0;
	  break;
	}
	while(count--) {cPCI_Zone_Fan_DAC_Register_Write(file_desc, param,param2);};
	break;
//--------------------------------------------------------------------
      case 0x27:
	if(nval <= 1) {
          printf("\n\n argument missing : data\n");
	  count = 0;
	  break;
	}
	while(count--) {cPCI_Calib_Led_Register_Read(file_desc, param);};
	break;
//--------------------------------------------------------------------
      case 0x28:
	if(nval <= 1) {
          printf("\n\n argument missing : data\n");
	  count = 0;
	  break;
	}
	while(count--) {cPCI_Calib_Led_Register_Write(file_desc, param,param2);};
	break;
//--------------------------------------------------------------------
	case 0x3b:
	  if(scr_level > 0) {
	    Script_File = dup(0);
	    close(0);
	    dup(indesc[0]);
	  }
	  if(tcgetattr(0, &confg_term) != 0) {
	    printf("\n error tcgetattr() : can't pause !\n");
	    if(scr_level > 0) {
	      close(0);
	      dup(Script_File);
	    }
	    break;
	  }
	  memcpy(&sauve_term, &confg_term, sizeof(struct termios));
	  cfmakeraw(&confg_term);
	  confg_term.c_cc[VMIN] = 0;
	  if(tcsetattr(0, TCSANOW, &confg_term) != 0) {
	    printf("\n error tcsetattr() : can't pause !\n");
	    if(scr_level > 0) {
	      close(0);
	      dup(Script_File);
	    }
	    break;
	  }
	  if(nval < 2) param = 15;
	  else if(param < 0) param = 15;

	  printf("\n\r execution is suspended : press any key to resume\r\n");

	  t1 = time(NULL);
	  t2 = t1;
	  while((t2 - t1) < param) {
	    if(read(0, mess, 1) == 1) break;
	    t2 = time(NULL);
	  }

	  tcsetattr(0, TCSANOW, &sauve_term);  
	  printf(" execution is resumed !\n");

	  if(scr_level > 0) {
	    close(0);
	    dup(Script_File);
	  }
	  last_val = SHOW_MENU;
	  break;

//--------------------------------------------------------------------

	  case 0x2c:
	  if(scr_level >= (SCRIPTS_RECURSIVITY)) {
	    printf("\n error : too much scripts recursivity (max %d)!\n",SCRIPTS_RECURSIVITY - 1);
	    break;
	  }
	  if(nval <= 1) {
	    pt = strtok(prm," #\a\r\n");
	    pt = strtok(NULL," #\a\r\n");
	    if(pt == NULL) {
	      printf("\n error : script name or number needed !\n");
	      break;
	    }
	    if(*pt == '#') {
	      printf("\n error : script name or number needed !\n");
	      break;
	    }
	    i = 0;
	    param  = nscripts;
	    while(i < nscripts) {
	      if(strcmp(pt, Script_name[i]) == 0) param = i;
		 i++;
	    }
	    if(param >= nscripts) {
	      printf("\n error : script name not found in Scripts_list_File : %s\n",FIFO_SCRIPT_LIST);
	      break;
	    }
	  }
	  if(param >= nscripts) { 
	    if(nscripts > 0) printf("\n error : invalid script number (should be < %x)\n",nscripts);
	    else printf("\n error : no script available !\n");
	    break;
	  }
	  strcpy(scrbuf,FIFO_SCRIPT_DIR);
	  strcat(scrbuf,"/");
	  strcat(scrbuf,Script_name[param]);

	  if((ScF = fopen(scrbuf, "r")) == NULL) {
	    //file doesn't exist yet 
	    printf ("\n error : script_file %s not found !\n",scrbuf);
	    break;
	  }
      	  setbuf(ScF,NULL);

	  strcpy(Script_Used_name[scr_level],Current_Script_name);
	  strcpy(Current_Script_name,Script_name[param]);
	  indesc[scr_level] = dup(0);
	  scr_level++;
	  close(0);
	  Script_File = fileno(ScF);
	  dup(Script_File);
	  close(Script_File);
	  printf("\n #############################################################");
	  printf("\n SYSTEM IS NOW EXECUTING SCRIPT_FILE %s\n",Current_Script_name);

	  last_val = SHOW_MENU;
	  break;
//--------------------------------------------------------------------
	case -1:
	  if(scr_level <= 0) {
	    close(file_desc);
	    close(Blk_File);  
	    printf("\n");
	    exit(0);
	  }
	  scr_level--;
	  close(0);
	  dup(indesc[scr_level]);
	  if(scr_level > 0) printf("\n SYSTEM IS BACK TO SCRIPT_FILE %s\n",Script_Used_name[scr_level]);
	  else printf("\n SYSTEM IS BACK TO KEYBOARD INPUT !\n");
	  printf(" #############################################################\n");
	  last_val = SHOW_MENU;
	  val = SHOW_MENU;
	  break;

//--------------------------------------------------------------------
	default:
	  if((prm[0] == '#') || (prm[1] == '#')) break;
	  if(scr_level > 0) break;
	  printf("\n\nusage is : %s [ FIFO_BLOCK_FILE_NAME ]\n",argv[0]);
	  display = 1;
	  break;
//--------------------------------------------------------------------
      }
	  
  }
getchar();
  close(file_desc);
  close(Blk_File);  
  exit(0);
}
